#!/usr/bin/env python
#
# Copyright 2009 Marcelo Boveto Shima.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA    02110-1301, USA.
#
# Author: marceloshima@gmail.com (Marcelo Boveto Shima)

import os, os.path
import sys
import subprocess
import logging

class TXPath:
    def __init__( self, sessionid=None, create=False, client=False ):
        if sessionid is None:
            if 'NXSESSIONID' not in os.environ:
                print( "Could not find the Session id" )
                sys.exit(1)
            sessionid = os.environ['NXSESSIONID']
	self.session_id = sessionid

        if client:
            self.session_dir = os.path.join(os.environ['HOME'], '.nx/S-' + self.session_id)
        else:
            self.session_dir = os.path.join(os.environ['HOME'], '.nx/C-' + self.session_id)
        if os.path.exists(self.session_dir):
            return

        if not create:
            print( 'Session path %s not found' % self.session_dir )
            sys.exit(1)

        os.makedirs( self.session_dir, 0700 )
        last_dir = os.path.join(os.environ['HOME'], '.nx/C-last')
        if os.path.lexists(last_dir):
            os.remove(last_dir)
        os.symlink(self.session_dir, last_dir)
        for dir in ('pids', 'logs', 'services'):
            os.makedirs(os.path.join(self.session_dir, dir), 0700)

    def get_logger( self, progname ):
        log_filename = os.path.join( os.path.join(self.session_dir, "logs"), progname + '.log')
        log_filename2 = os.path.join( os.path.join(self.session_dir, "logs"), 'all.log')
        return get_logger( progname, log_filename, log_filename2  )

    def get_filepath( self, filename ):
        return os.path.join( self.session_dir, filename )

    def get_dir( self ):
        return self.session_dir

    def get_failed_dir( self ):
        return os.path.join(os.environ['HOME'], '.nx/F-' + self.session_id)

    def get_terminated_dir( self ):
        return os.path.join(os.environ['HOME'], '.nx/T-' + self.session_id)

    def get_unknown_dir( self ):
        return os.path.join(os.environ['HOME'], '.nx/U-' + self.session_id)

    def set_pid(self, pid_name, pid):
        self.write_file('pids/' + pid_name, pid)

    def get_pid(self, pid_name):
        file = os.path.join(self.session_dir, 'pids/' + pid_name)
        pid = open(file, 'r').read().strip()
        return int(pid)

    def set_port(self, service_name, port):
        self.write_file('services/' + service_name, port)

    def write_file(self, file_name, info):
        file = os.path.join(self.session_dir, file_name)
        open(file, 'w').write(str(info))

def opt_read(opt_str, sep=","):
    """Parse options into a dict."""

    options = {}
    if opt_str.find( sep ) is -1:
        return options
    for pair in opt_str.split( sep ):
        if pair.find( '=' ) is not -1:
            name, val = pair.split('=')
            options[name] = val
    return options

def exists_path(session_id):
    session_dir = os.path.join(os.environ['HOME'], '.nx/C-' + session_id)
    if os.path.exists(session_dir):
        return True
    return False

def get_tmp_logger( progname ):
    log_filename = os.path.join( "/tmp", progname + '.log')
    return get_logger( progname, log_filename )

def get_home_logger( progname ):
    log_filename = os.path.join( os.environ['HOME'], '.nx/' + progname + '.log')
    return get_logger( progname, log_filename )

def get_logger( progname, file, file2=None ):
    logger = logging.getLogger (progname)
    logger.setLevel(logging.DEBUG)
    handler = logging.FileHandler(file)
    formatter = logging.Formatter('%(asctime)s %(name)-12s %(levelname)-8s %(message)s')
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    if file2 is not None:
        handler2 = logging.FileHandler(file2)
        handler2.setFormatter(formatter)
        logger.addHandler(handler2)

    logger.debug ("Starting %s" % progname)
    return logger

def add_cookie_to_xauthority(authority_file, display, cookie):
    command = ['xauth', '-f', authority_file, 'add', display, 'MIT-MAGIC-COOKIE-1', cookie]
    return exec_xauth(authority_file, command)

def remove_display_from_xauthority(authority_file, display):
    command = ['xauth', '-f', authority_file, 'remove', display]
    return exec_xauth(authority_file, command)

def exec_xauth(authority_file, command):
    statinfo = None
    # Backup the file attributes if the file exists
    if os.path.lexists(authority_file):
        statinfo = os.stat(authority_file)
    retcode = subprocess.call(command, stdout=None, stderr=None)
    # Revert to original owner
    if statinfo is not None:
        os.lchown(authority_file, statinfo.st_uid, statinfo.st_gid)
    if retcode == 0:
        return True
    return False

def merge_cookie_from_xauthority(source_file, dest_file, display):
    statinfo = None
    # Backup the file attributes if the file exists
    if os.path.lexists(dest_file):
        statinfo = os.stat(dest_file)
    cmd = 'xauth -f "%s" extract - %s | xauth -f "%s" merge -' % (source_file, display, dest_file)
    retcode = subprocess.call(cmd, shell=True, stdout=None, stderr=None)
    # Revert to original owner
    if statinfo is not None:
        os.lchown(dest_file, statinfo.st_uid, statinfo.st_gid)
    if retcode == 0:
        return True
    return False

def run_program(program, authority_file, display):
    environ = {}
    environ.update(os.environ)
    environ['DISPLAY'] = display
    environ['XAUTHORITY'] = authority_file
    ret = subprocess.call(program, shell=True, stdout=None, stderr=None, env=environ)
    return ret


