#!/usr/bin/python
#
# Copyright 2009 Marcelo Boveto Shima.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA    02110-1301, USA.
#
# Author: marceloshima@gmail.com (Marcelo Boveto Shima)

import os
import string
import NXShares
import NXSessionType

class NXSession:
    def __init__( self ):
        self.nxsessionid = os.getenv( 'NXSESSIONID' )
        if self.nxsessionid is None:
            return
        
        self.session_dir = os.getenv( 'HOME' ) + '/.nx/C-' + self.nxsessionid
        self.agent_file = self.session_dir + '/pids/agent'
        self.mpoint_file = self.session_dir + '/scripts/mpoint'
        self.nx_session_type = NXSessionType.GNOMENXSessionType( )
        self.shares = NXShares.NXShares( self, self.nx_session_type )

    def browse_dir( self ):
        self.nx_session_type.browse_dir( self.session_dir )
    
    def exists( self ):
        if self.nxsessionid is None:
            return False
        return True
    
    def get_mounts_file_path( self ):
        return self.mpoint_file
    
    def has_mounts( self ):
        if os.path.isfile( self.mpoint_file ):
            return True
        return False
    
    def get_agent_pid( self ):
        if not os.path.isfile( self.agent_file ):
            return
        agent = open( self.agent_file,'r' )
        line = agent.readline( )
        agent.close( )
        line = string.strip( line )
        if line is not '':
            return line
        else:
            return None
    
    def send_signal ( self, signal ):
        agent_pid = self.get_agent_pid( )
        if agent_pid is None:
            print "Unable to find the agent PID"
            return
        ret = os.spawnlp( os.P_WAIT, "kill", "kill", signal, agent_pid )
        if ret is not 0:
            print "Error sending " + signal + " to the session"
    
    def suspend( self ):
        self.send_signal( "-HUP" )
    
    def terminate( self ):
        self.send_signal( "-TERM" )
    
    def umount_all( self ):
        self.shares.umount_all( )
