#!/usr/bin/env python

import os
import sys
import dbus

import tacix.nx.NXSession
import tacix.TXUtils
from tacix.freenx.FreeNXNodeProtocol import FreeNXNodeProtocol
from tacix.nx.NXNode import NXNode
from tacix.freenx.FreeNXShell import FreeNXException

BUS_NAME="org.tacix"
INTERFACE_SESSION="org.tacix.Sessions"
INTERFACE_SERVER="org.tacix.Manager"
OBJECT_PATH="/org/tacix/Manager"


class FreeNXNode(NXNode, FreeNXNodeProtocol):
    def __init__( self, logger=None ):
        FreeNXNodeProtocol.__init__(self)
        NXNode.__init__(self, logger=logger)

    # Watchers
    def watch_agent_state_resume(self, state):
        if state == 'running':
            self.say_session_resumed()
            self.say_bye()
            self.loop.quit()
        elif state == 'failed':
            self.say_session_resume_failed()
            self.say_bye()
        else:
            self.watch_agent_state(state)

    def watch_agent_state(self, state):
        # On session supended remove signals
        if state == 'suspended':
            sys.stdout.write('NX> 1005 Session status: %s\n' % state)
            self.say_bye()
            sys.stdout.flush()
            self.remove_signals()

        elif state == 'waiting_connection':
            self.say_connected()
            self.say_waiting()

        elif state == 'denied':
            self.say_abort('Permission denied')
            self.say_bye()
        else:
            sys.stdout.write('NX> 1009 Session status: %s\n' % state)

            if state == 'terminated':
                self.say_closed()
                self.say_bye()
            elif state == 'failed':
                self.say_error()
                self.say_bye()

        sys.stdout.flush()
        NXNode.watch_agent_state(self, state)

    # Actions
    def list_session(self, options):
        try:
            self.say_listsessions_header(options)
            if options['type'] == 'shadow':
                sessions = self.manager.GetShadowableSessions()
                for session_key in sessions.keys():
                    self.say_listsessions_sessions(sessions[session_key])

            else:
                sessions = self.manager.GetResumableSessions (options)
                for session_opts in sessions.values():
                    self.logger.debug(session_opts)
                    self.say_listsessions_sessions(session_opts)
        except dbus.DBusException, e:
            self.logger.debug( 'DBusError: %s' % e)
            raise FreeNXException( e._dbus_error_name )

        self.say_listsessions_footer(options)

