#!/usr/bin/env python

import os
import sys
import gobject
import re
import subprocess
import time

import tacix.TXUtils
from tacix.freenx.FreeNXNode import FreeNXNode
from tacix.freenx.FreeNXNode import FreeNXNode
from tacix.freenx.FreeNXServerProtocol import FreeNXServerProtocol
from tacix.freenx.FreeNXShell import *

class FreeNXNodeSlave(FreeNXNode, FreeNXShell, FreeNXServerProtocol):
    def __init__( self, logger=None, txpath=None ):
        FreeNXShell.__init__(self, logger=logger)
        FreeNXNode.__init__(self, logger=logger)

        self.txpath = txpath
        if self.logger is None and self.txpath is not None:
            self.txpath.get_logger('freenxnodeslave')

    # Helpers
    def _startsession_handler(self, line):
        opts = self.opt_read(line)
        self.start_session(opts)
        return True

    def _listsession_handler(self, line):
        opts = self.opt_read(line)
        if self.logger is not None:
            self.logger.debug('Listing sessions %s' % line)

        self.list_session(opts)
        return True

    def _restoresession_handler(self, line):
        opts = self.opt_read(line)
        self.restore_session(opts)
        return True

    def _attachsession_handler(self, line):
        print("NX> 726 Asking user for authorization to attach to session")
        sys.stdout.flush()
        opts = self.opt_read(line)
        self.attach_session(opts)
        self.wait_session()
        return True

    def _addprinter_handler(self, line):
        #['addprinter  --type="ipp" --username="mshima" --port="3001" --session_id="4022DFDCEAAA1829A1AFA33124D881BC" --printer="IBM-Compatible_Dot_Matrix" --password="b89f37c8ac10dea3" --model="NULL"\n']
        opts = self.opt_read(line)
        self.add_printer(opts)
        return True

    def _terminate_handler(self, line):
        opts = self.opt_read(line)
        # Terminate send sessionid instead of id so copy it
        opts['id'] = opts['sessionid']
        self.terminate_session(opts)
        return True

    def _check_handler(self, line):
        self.check()
        sys.stdout.flush()
        return True

    def _quit_handler(self, line):
        return False

    def _bye_handler(self, line):
        self.say_server_bye(line)
        self.logger.debug('Waiting session to finish')
        self.wait_session()

    def slave_loop(self):
        '''
        Init the FreeNX Node in slave mode
        '''
        self.say_hello()
        self.say_slave_started()
        sys.stdout.flush()
        gobject.io_add_watch(sys.stdin, gobject.IO_IN | gobject.IO_HUP, self.read_line)
        self.wait_session()

    def read_line(self, source, condition):
        if condition == gobject.IO_HUP:
            return False
        self.set_non_blocking(sys.stdin.fileno())
        ret = False
        try:
            ret = self.receive_stdin()
        except Exception, e:
            self.say_abort(e)
        self.revert_blocking(sys.stdin.fileno())
        return ret


