#!/usr/bin/python
#
# Copyright 2009 Marcelo Boveto Shima.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA    02110-1301, USA.
#
# Author: marceloshima@gmail.com (Marcelo Boveto Shima)

import os
import shutil
import traceback
import subprocess
import dbus
import gobject
gobject.threads_init()

from tacix.nx.NXAgent import NXAgent
from tacix.nx.NXSessionInterface import NXSessionInterface
from tacix.TXUtils import TXPath

BUS_NAME="org.tacix"
INTERFACE_SESSION="org.tacix.Sessions"
INTERFACE_SERVER="org.tacix.Manager"
OBJECT_PATH="/org/tacix/Manager"

class NXSessionResources(NXSessionInterface):
    def __init__ ( self, sessionid ):
        from dbus.mainloop.glib import DBusGMainLoop
        DBusGMainLoop(set_as_default=True)
        self.loop = gobject.MainLoop()

        self.bus = dbus.SystemBus()
        manager_obj = self.bus.get_object (BUS_NAME, OBJECT_PATH)
        self.manager = dbus.Interface (manager_obj, INTERFACE_SERVER)
        session_path = self.manager.GetSession (sessionid)
        session_obj = self.bus.get_object (BUS_NAME, session_path)

        self.txpath = TXPath( sessionid, create=True )

        self.txsession = dbus.Interface (session_obj, INTERFACE_SESSION)
        NXSessionInterface.__init__(self,
                self.txsession,
                NXAgent(self, self.txsession.GetProperties()),
                self.txpath,
                self.txpath.get_logger("nxsession"))

        self.pending_printers = []

    # Watchers
    def watch_agent_state_resources(self, state):
        if state == 'running':
            self.logger.debug('Printers %s' % self.pending_printers)
            for printer_name in self.pending_printers:
                self.add_printer(printer_name)
            self.pending_printers = []
            self.logger.info("Nxagent state is %s" % state)
            if 'mediahelper' in self.nxagent.options:
                gobject.timeout_add(1, self.init_sound)
        elif state == 'terminating' or state == 'suspending':
            if 'mediahelper' in self.nxagent.options:
                self.cleanup_sound()

    def watch_printer_solicited(self, printer_name):
        self.logger.debug('Printer %s solicited' % printer_name)
        if self.txsession.GetState() != 'running':
            self.logger.debug('Appending printer %s to solicited printers' % printer_name)
            self.pending_printers.append(printer_name)
            return
        self.add_printer(printer_name)

    # Action methods
    def setup_signal_resources(self):
        try:
            if self.nxagent.options['media'] == '1' or self.nxagent.options['cups'] == '1':
                self.resources_signal = self.txsession.connect_to_signal("StateChanged", self.watch_agent_state_resources, dbus_interface=INTERFACE_SESSION)
            
            if self.nxagent.options['cups'] == '1':
                self.logger.debug('Printer watcher added')
                self.printersolicited_signal = self.txsession.connect_to_signal("PrinterSolicited", self.watch_printer_solicited, dbus_interface=INTERFACE_SESSION)
            self.logger.debug('Watcher for resources added')
        except Exception, e:
            self.logger.error("Got exception, cleaning up\n")
            trace = traceback.format_exc()
            for line in trace.split('\n'):
                self.logger.error('%s' % line)

    def init_sound(self):
        if self.nxagent.options['mediahelper'] == 'pulse':
            self.logger.debug('Adding sink to pulseaudio')
            ret = subprocess.call('pactl load-module module-esound-sink sink_name=nx', shell=True)
            if ret == 0:
                self.logger.debug('PulseAudio configured')
                return False
            self.logger.debug('PulseAudio configuration failed')
            return True

    def setup_sound(self):
        if 'mediahelper' in self.nxagent.options:
            if self.nxagent.options['mediahelper'] == 'pulse':
                os.environ['ESPEAKER'] = '127.0.0.1:%s' % str(int(self.nxagent.options['display']) + 7000)

    def cleanup_sound(self):
        if self.nxagent.options['mediahelper'] == 'pulse':
            pass

    def mount_share(self):
        #mkdir -p "$HOME/$dir"
        # wait up to 30 sec until nxagent has opened the listener port for samba connections
        #for (( i=30; $i; --i ))
        #do
        #        [ -f "$USER_FAKE_HOME/.nx/C-$SERVER_NAME-$display-$sessionid/scripts/mport" ] && break
        #        sleep 1
        #done
        # sometimes the samba port we get from nxserver is not the right one, so let's get it from nxagent
        #realport=$(cat "$USER_FAKE_HOME/.nx/C-$SERVER_NAME-$display-$sessionid/scripts/mport")
        #[ -n "$realport" ] && port=$realport

        #error=$(PASSWD="$password" $COMMAND_SMBMOUNT "//$computername/$share" "$HOME/$dir" -o username="$username",ip=127.0.0.1,port=$port 2>&1)
        #if [ $? -eq 0 ]
        #then
        #$PATH_BIN/nxdialog -dialog ok -caption "NXServer Message" -message "Info: Share: '//$computername/$share' mounted on: '$HOME/$dir'" -display :$display &
        #echo "$HOME/$dir" >> "$USER_FAKE_HOME/.nx/C-$SERVER_NAME-$display-$sessionid/scripts/mpoint"
        #else
        #$PATH_BIN/nxdialog -dialog ok -caption "NXServer Message" -message "Info: Share: '//$computername/$share' failed to mount: $error" -display :$display &
        pass

    def kill(self, pid, signal):
        return True

    def cleanup_printer_daemon(self):
        # if the daemon is not running abort
        cups_pid = self.txsession.GetProperty('cups_pid')
        if cups_pid is None or cups_pid == '':
            return False

        if not self.kill(int(cups_id), 0):
            self.txsession.RemovePrinterDaemon()

        self.kill(cups_pid, signal.SIGTERM)
        time.sleep(2)
        if self.kill(cups_pid, 0):
            self.kill(cups_pid, 9)

        self.txsession.RemovePrinterDaemon()
        # remove all printers
        #echo >"$USER_FAKE_HOME/.nx/C-$sess_id/cups/printers.conf"

    def setup_printer_daemon(self):
        if False:
            cupsd_port = int(self.txsession.GetProperty('display')) + 9000
            os.environ['CUPS_SERVER'] = "http://localhost:%s" % cupsd_port
            return

        session_id = self.txsession.GetProperty('id')

        os.environ['CUPS_SERVER'] = self.txpath.get_filepath("cups/cups.sock")
        
        for dir in ('cups/spool/tmp', 'cups/spool/certs', 'cups/ppd', 'cups/cache', 'cups/logs'):
            os.makedirs(self.txpath.get_filepath(dir), 0700)

        cupsd_conf_template = '/etc/tacix/cups.conf.template'
        prop = {'session_dir': self.txpath.get_dir()}
        cupsd_conf = self.txpath.get_filepath('cups/cupsd.conf')
        confs = open(cupsd_conf_template, 'r').read()
        open(cupsd_conf, 'w').write(confs % prop)

        open(self.txpath.get_filepath('cups/printers.conf'), 'w').write('')
        open(self.txpath.get_filepath('cups/classes.conf'), 'w').write('')

        # Is this needed??
        # copy mime.* files
        shutil.copy('/etc/cups/mime.convs', self.txpath.get_filepath('cups/'))
        shutil.copy('/etc/cups/mime.types', self.txpath.get_filepath('cups/'))

        # start cupsd
        cups_output = open(self.txpath.get_filepath('cups/cups.log'), 'w')

        # On ubuntu /usr/sbin/cupsd is app_armor profiled, don't allows to read /home/*
        cupsd_cmd = '/usr/lib/tacix/cupsd'

        subprocess.Popen([cupsd_cmd, '-c', cupsd_conf], stdin=None, stdout=cups_output, stderr=cups_output)

        # This should go to somewhere else
        # setup KDE
        #if [ "$ENABLE_KDE_CUPS" = "1" -a -e "$KDE_PRINTRC" ]
        #then
        #    if egrep -q "^Host=" "$KDE_PRINTRC"
        #    then
        #        [ "$ENABLE_KDE_CUPS_DYNAMIC" = "1" ] && $COMMAND_PERL -pi -e 's,^Host=.*,Host[\$ie]=\$\('"$PATH_BIN/nxcups-gethost"'),g' "$KDE_PRINTRC"
        #        [ "$ENABLE_KDE_CUPS_DYNAMIC" != "1" ] && $COMMAND_PERL -pi -e 's,^Host=.*,Host='"$NODE_CUPSD_SOCKET"',g' "$KDE_PRINTRC"
        #    else
        #        echo "[CUPS]" >> "$KDE_PRINTRC"
        #        [ "$ENABLE_KDE_CUPS_DYNAMIC" = "1" ] && echo "Host[\$ie]=\$($PATH_BIN/nxcups-gethost)" >> "$KDE_PRINTRC"
        #        [ "$ENABLE_KDE_CUPS_DYNAMIC" != "1" ] && echo "Host=$NODE_CUPSD_SOCKET" >> "$KDE_PRINTRC"
        #    fi
        #fi

    def reload_printer_configs(self):
        cups_pid = self.txsession.GetProperty('cups_pid')
        if cups_pid is None or cups_pid == '':
            return False

        if not self.kill(int(cups_id), 0):
            return False

        self.kill(int(cups_id), signal.SIGHUP)
        return True

    def add_printer(self, printer_name):
        self.logger.debug('Adding printer %s' % printer_name)
        printer = self.txsession.GetPrinter(printer_name)
        self.logger.debug('Printer parameters %s' % printer)
        #printer = {'type': 'smb', 'username':'user', 'password':'password', 'port':'port', 'share':'share', 'printer':'printer'}

        if 'type' not in printer:
            self.logger.error('Printer type not found')
            return

        if printer['type'] == 'smb':
            device_uri = "smb://%(username)s:%(password)s@127.0.0.1:%(port)s/%(share)s" % printer
            device_name = printer['share']
        else:
            device_uri = "ipp://%(username)s:%(password)s@127.0.0.1:%(port)s/printers/%(printer)s" % printer
            device_name = printer['printer']

        model = printer['model']

        cmd = ['/usr/sbin/lpadmin', '-p', device_name, '-E' ,'-v', device_uri]

        ppd_file = None
        if ppd_file is not None:
            cmd.extend(['-P', ppd_file])
        else:
            if model.lower() != 'null':
                cmd.extend(['-m', model])

        if 'public' not in printer or printer['public'] != '1':
            cmd.extend(['-u', 'allow:%s' % self.txsession.GetProperty('user')])

        lpadmin_log = open(self.txpath.get_filepath('cups/lpadmin.log'), 'w')
        self.logger.debug('Calling lpadmin %s' % cmd)
        subprocess.call(cmd, stdin=None, stdout=lpadmin_log, stderr=lpadmin_log)

        if 'defaultPrinter' in printer and printer['defaultPrinter'] == '1':
            subprocess.call(['/usr/sbin/lpadmin', '-d', device_name], stdin=None, stdout=lpadmin_log, stderr=lpadmin_log)

        #subprocess.call(['/usr/bin/system-config-printer', '--choose-driver', device_name], stdin=None, stdout=lpadmin_log, stderr=lpadmin_log)

        #if [ "$ENABLE_CUPS_SEAMLESS" != "1" ]
        #then
        #    MODEL=$($PATH_BIN/nxdialog -printer "$NAME" -display :$display)
        #    [ -z "$MODEL" -o "$MODEL" = "cancel: aborted" ] && return
        #else
        #    sleep $CUPS_SEAMLESS_DELAY
        #    MODEL="download_cached"
        #fi
        
        #if [ "$MODEL" = "download_new" -o "$MODEL" = "download_cached" ]
        #then
        #    mkdir -p "$USER_FAKE_HOME/.nx/config/ppd/"
        #    if [ "$MODEL" = "download_new" -o ! -r "$USER_FAKE_HOME/.nx/config/ppd/$printer.ppd" ]
        #    then
        #        curl --proxy "" --digest -o "$USER_FAKE_HOME/.nx/config/ppd/$printer.ppd" "http://$username:$password@127.0.0.1:$port/ppd/${printer}_nxdl.ppd" >>"$USER_FAKE_HOME/.nx/C-$sess_id/cups/curl.log" 2>&1
        #    RC=$?
        #    [ $RC -eq 0 -a "$ENABLE_CUPS_SEAMLESS" != "1" ] && $PATH_BIN/nxdialog --message "Download successful." -display :$display
        #    [ $RC -ne 0 -a "$ENABLE_CUPS_SEAMLESS" != "1" ] && $PATH_BIN/nxdialog --message "Download failed. Log is: $USER_FAKE_HOME/.nx/C-$sess_id/cups/curl.log" -display :$display
        #    fi
        #    MODEL="$USER_FAKE_HOME/.nx/config/ppd/$printer.ppd"
        #fi

